from RciPacket import RciPacket
from RciLog import RciLog

# Reply packet will be generated at the server and will be sent to the client as an answer to a request packet
class RciReplyPacket(RciPacket):
    
    RCI_REPLY_NBITS = 8
    RCI_REPLY_ID = {
                    'RdRqReply'     : 0x0000,
                    'WrRqSuccess'   : 0x0001,
                    'WrRqFailed'    : 0x0002,
                    'CmdRqSuccess'  : 0x0003,
                    'CmdRqFailed'   : 0x0004,
                    'ProtocolErr'   : 0xFFFF
                    }
    RCI_REPLY_PROT_ERROR = {
                            'IllegalStartOfFrame'   : 0x00,
                            'VersionNotSupported'   : 0x01,
                            'PacketTypeIdUnkown'    : 0x02,
                            'RequestBaseIdUnkow'    : 0x03,
                            'RequestSubIdUnown'     : 0x04
                            }
    log = RciLog("RciReplyPacket")

    def __init__(self):
        RciPacket.__init__(self,'RCI_REPLY_PID')
        #super(RciReplyPacket,self).__init__(self)
    
    def getReplyId(self):
        return self.getPacketDescriptor() & 0xFFFF
 
    def getReplyType(self):
        for key in self.RCI_REPLY_ID.keys():
            if (self.getPacketDescriptor() & 0xFFFF) == self.RCI_REPLY_ID[key]:
                return key
                break
        return None
       
    def setReplyPacketDescriptor(self, rid):
        try: 
            if rid < self.RCI_REPLY_ID['RdRqReply'] or rid > self.RCI_REPLY_ID['ProtocolErr']:
                raise ValueError
            self.setPacketDescriptor(rid)
        except ValueError:
            self.log.write("Reply ID out of bounds",'ERROR')
            raise ValueError("setReplyPacketDescriptor: Reply ID out of bounds")

    def getProtocolErrorType(self,err_code):
        for key in self.RCI_REPLY_PROT_ERROR.keys():
            if self.RCI_REPLY_PROT_ERROR[key] == err_code:
                return key
        return None

    def setProtocolError(self,err_code):
        self.setReplyId(self.RCI_REPLY_ID['ProtocolErr'])
        self.setNumBytesPayload(1)
        self.payload[0] = err_code
        